'use strict'
var config = {

    // 新手参数 Start
    // 端口 - Port: http://127.0.0.1:18080/
    // Example: 18080 or 8888 or 8080
    port: 18080,

    // 默认为 app.js 所在目录下的所有文件均为静态资源文件
    static: process.cwd(),

    // 首页中 HTML list 读取的目录 - Home page HTML list read directory
    html: [],

    // markdown/html 文档的保存地址
    docPath: {
        html: '/docs/data-api.html',
        markdown: '/docs/data-api.md'
    },

    // 新手参数 End

    // express 中的 connect，如不熟悉请无视
    connect: function (req, res ,next) {
        next();
    },

    // 访问密码 - Access to the password
    // Example:  false or "nodeajax"
    // 参数是 false 则不需要密码 - If parameter is false, do not need a password
    password: false,
    // URL重定向 - Rewrite rule
    urlRewrite: [
        // ['/', '/html/index.html']
    ],

    ajaxSettings: {
        type: 'get',
        // // 请求参数
        // data: {
        //     id: 1
        // }
        success: {
            status: 'success'
        },
        error: {
            status: 'error',
            msg: 'Error detail'
        },
        doc: function() {/*!
{{title}}
---------
- url: [{{url}}]({{url}})
- type: `{{type}}`

{{#if request}}
#### Request:
{{request}}
{{/if}}
#### Response:
{{#if success}}
**success:**  
```js
{{success}}
```
{{/if}}
{{#if error}}
**error:**  
```js
{{error}}
```
{{/if}}
        */},
        statusCode: [200, 200],
        timeout: 300,
        dataType: 'html'
    },
    
    errorPage: '/html/404.html',

    // 静态资源替换 - Static write replace
    staticWriteReplace: function (file, req) {
        // 一定要同步操作 existsSync readFileSync
        // file = {
        //     pathname: pathname ( example/index.html )
        //     dirname: dirname (example/)
        //     data: filedata ( <!DOCTYPE html> <html lang="en"> <head> ... </html> )
        // }

        // 显示特定注释中的代码
        // <!--nodeajax
        // <strong>text</strong>
        // nodeajax-->
        // 将会被替换为如下代码
        // <strong>text</strong>
        file.data = file.data.replace(/<\!--nodeajax([\s\S]*)?nodeajax-->/gm, '$1');

        // 原生PHP include 语法支持
        // <?php include "common/header"; ?>
        // content
        // <?php include "common/footer"; ?>
        file.data = file.data.replace(/<\?php (include|require) ["]([^"]+)"; \?>/g, function () {
            var sPath = arguments[2],
            fs = require('fs');
            var sPath = file.dirname + sPath
            if (!fs.existsSync(sPath)) {
                return '<strong style="color:#333;background:#f2dede;">ENOENT, no such file ' + sPath + '</strong>'; 
            } else {
                try {
                    return fs.readFileSync(sPath, 'utf-8');
                } catch (err) {
                    return err.message;
                }
            }
        })
        
    },
    pageSettings: {
        data: {
            /*
            DEV:true,
            APP_PATH: '/',
            USER: {
                id: 23,
                name: 'nimo',
                email: 'nimo.jser@gmail.com',
                avatar: 'https://avatars1.githubusercontent.com/u/3949015?v=3&s=50'
            }
            */
        },
        // render 可在渲染前操作数据源，非全局，每个页面单独操作
        // 渲染执行顺序:2
        render: function (options, req) {
            
        },        
        doc: function() {/*!
{{title}}
---------
- url: [{{url}}]({{url}})

{{#if request}}
#### Request:
{{request}}
{{/if}}
#### Data:
{{#if data}}
**success:**  
```js
{{data}}
```
{{/if}}

        */}
    },
    page: {
        helper: {
            
        },
        // 在 page 文档中过滤这些选项
        docDataFilter: ['DEV','APP_PATH'],
        // req 用于向数据源提供 http 相关信息。全局性，每次渲染都会执行
        // 渲染执行顺序:1
        req: function (options, req) {
            /*
            options.data.PAGE_URL = req.url;
            options.data.PAGE_PATH = req.path;
            options.data.METHOD = req.method;
            options.data.GET = req.query;
            options.data.POST = req.body;
            */
        },
        // 对模板引擎做二次封装。全局性，对渲染拥有最终控制权
        // 可用于扩展模板引擎
        // 渲染执行顺序:3
        globalRender: function (options, req) {
            // <?php tpl_include('templatepath.html'); ?> 的扩展语法实现
            /*
            var sDirname = options.template.replace(/[^/]+$/, '');
            if (!/^\//.test(sDirname)) {
                sDirname = '/' + sDirname;
            }
            options.source = options.source.replace(/<\?php tpl_include\((["'])([^"']+)\1\); \?>/g, function () {
                var sPath = arguments[2],
                fs = require('fs'),
                sFilepath = __dirname + sDirname + sPath,
                sResult;
                
                if (!fs.existsSync(sFilepath)) {
                    sResult = '<strong style="color:#333;background:#f2dede;">ENOENT, no such file ' + sDirname + sPath + '</strong>'; 
                } else {
                    try {
                        sResult = fs.readFileSync(sFilepath, 'utf-8');
                    } catch (err) {
                        sResult = err.message;
                    }
                }
                return sResult
            })
            */
        }
    }
}
module.exports = config