var colors = require('colors')
var fs = require('fs')
var _util = require('./util')
var hljs = require('highlight.js')
var marked = require('marked')
var doc = require('./doc')
var fse = require('fs-extra')
var init = function (settings) {
    var $ = require('../index')
    var server = require('./server')
    var defaultConfig = require('./config')
    
    
    if (typeof settings !== 'object') {
        settings = {
            port: settings
        }
    }

    
    $.extend(true, defaultConfig, settings)
    
    $.config = defaultConfig

    $.config.port = process.env.PORT || $.config.port

    $.config.html.push('/')
    $.config.html = $._.uniq($.config.html)

    $.config.urlRewrite.forEach(function (value, index, arr) {
        var searchValue = value[0]
        if (typeof searchValue === 'string') {
            if (searchValue.charAt(0) !== '/' && searchValue.length !== 0) {
                var newValue = '/' + searchValue
                arr[index][0] = newValue
                console.log(('"' + searchValue + '" changed to "' + newValue + '"').yellow)
                // Ref: https://github.com/nimojs/nodeajax/issues/9
            }
        }
        
    })
    $._.each($.config.page.helper, function (fn, name) {
        $.Handlebars.registerHelper(name, fn)
    })

    // setTimeout 用于防止 app.js 中的 $.ajax 比当前代码后执行导致的此时 doc._data 为空
    setTimeout(function () {
        var aMarkdownDoc = []
        var sMarkdownDoc = ''
        doc._data.forEach(function (value) {
            switch (value.type) {
                case 'ajax':
                    /*!
                        oRenderData: 是 $.ajax 中 settings 的变种
                        它完全复制了 settings 并且为了成为文档渲染的渲染数据源做了一些优化
                        例如: oRenderData.title = oRenderData.title || oRenderData.url;
                     */
                    var settings = value.settings
                    var oRenderData = $.extend(true, {}, settings)

                    var request = oRenderData.request || false
                    if (typeof request === 'object') {
                        request = '```js\r\n' + _util.formatJson(JSON.stringify(request)) + '\r\n```'
                    }
                    oRenderData.request = request
                    oRenderData.title = oRenderData.title || oRenderData.url
                    if (typeof settings.doc === 'function') {
                        settings.doc = $.heredoc(settings.doc)
                    }

                    var fTransitionResult = function (target) {
                        switch (typeof(target)) {
                            case 'object':
                                if (settings.mock) {
                                    target = Mock.mock(target);
                                }
                                target = _util.formatJson(JSON.stringify(target))
                            break;

                            case 'number':
                                target = String(target);
                            break;

                            case 'function':
                            target = null;
                            break;

                            default:
                        }
                        return target;
                    };

                    oRenderData.success = fTransitionResult(settings.success)
                    oRenderData.error = fTransitionResult(settings.error)

                    // {{success}} to {{&success}}
                    settings.doc = settings.doc.replace(/{{(success|error|url|title|data)}}/g,'{{&$1}}');

                    // <!--#[/ajax/demo/]-->
                    // 此处是为了 dashbaord 在浏览器中进行替换 
                    // html = html.replace(/\s*<!--#\[([^[]+)]-->\s*/g,'<a href="#$1" class="ui-view-anchor" name="$1">#</a>')
                    // Result: <a href="#/ajax/demo/" class="ui-view-anchor" name="/ajax/demo/">#</a>
                    aMarkdownDoc.push('<!--#['+ oRenderData.url +']-->\r\n');

                    aMarkdownDoc.push($.Handlebars.compile(settings.doc)(oRenderData))
                break
                case 'page':
                    var settings = value.settings
                    var oRenderData = $.extend(true, {}, settings)

                    var request = oRenderData.request || false
                    if (typeof request === 'object') {
                        request = '```js\r\n' + _util.formatJson(JSON.stringify(request)) + '\r\n```'
                    }
                    oRenderData.request = request
                    oRenderData.title = oRenderData.title || oRenderData.url
                    if (typeof settings.doc === 'function') {
                        settings.doc = $.heredoc(settings.doc)
                    }

                    $._.each($.config.page.docDataFilter, function (name) {
                        delete oRenderData.data[name]
                    })
                    oRenderData.data = _util.formatJson(JSON.stringify(oRenderData.data))

                    settings.doc = settings.doc.replace(/{{(success|error|url|title|data)}}/g,'{{&$1}}');

                    // <!--#[/ajax/demo/]-->
                    // 此处是为了 dashbaord 在浏览器中进行替换 
                    // html = html.replace(/\s*<!--#\[([^[]+)]-->\s*/g,'<a href="#$1" class="ui-view-anchor" name="$1">#</a>')
                    // Result: <a href="#/ajax/demo/" class="ui-view-anchor" name="/ajax/demo/">#</a>
                    aMarkdownDoc.push('<!--#['+ oRenderData.url +'!]-->\r\n');

                    aMarkdownDoc.push($.Handlebars.compile(settings.doc)(oRenderData))
                break
                case 'template':
                    aMarkdownDoc.push($.Handlebars.compile(value.template)(value.renderData))
                break
                default:
            }
        })
        aMarkdownDoc.push('<!-- Created by www.nodeajax.org  -->')
        sMarkdownDoc = aMarkdownDoc.join('\r\n \r\n')
        

        

        marked.setOptions({
            highlight: function (code, lang, callback) {
                // Ref: https://github.com/nimojs/nodeajax/issues/27
                if (lang) {
                    return hljs.highlight(lang, code).value
                }
                else {
                    return hljs.highlightAuto(code).value
                }
            }
        })
        
        $._data.doc_html = marked(sMarkdownDoc)

        var docHtmlTemplate = fs.readFileSync(__dirname + '/doc-html.hbs', 'utf-8')
        var html = $.Handlebars.compile(docHtmlTemplate)({
            content: $._data.doc_html
        })
        if ($.config.docPath) {
            // 生成 markdown格式 文档
            if ($.config.docPath.markdown) {
                fse.outputFileSync($._cwd + $.config.docPath.markdown, sMarkdownDoc)
            }
            // 生成 html格式 文档
            if ($.config.docPath.html) {
                fse.outputFileSync($._cwd + $.config.docPath.html, html)
            }
        }
        
    }, 0)
    server()
}
module.exports = init